import { RawClient } from "../generated/client.js";
import { WebsocketTransport, BaseTransport } from "yerpc";
import { TinyEmitter } from "@deltachat/tiny-emitter";
export class BaseDeltaChat extends TinyEmitter {
    constructor(transport, startEventLoop) {
        super();
        this.transport = transport;
        this.contextEmitters = {};
        this.rpc = new RawClient(this.transport);
        if (startEventLoop) {
            this.eventTask = this.eventLoop();
        }
    }
    async eventLoop() {
        while (true) {
            const event = await this.rpc.getNextEvent();
            //@ts-ignore
            this.emit(event.event.kind, event.contextId, event.event);
            this.emit("ALL", event.contextId, event.event);
            if (this.contextEmitters[event.contextId]) {
                this.contextEmitters[event.contextId].emit(event.event.kind, 
                //@ts-ignore
                event.event);
                this.contextEmitters[event.contextId].emit("ALL", event.event);
            }
        }
    }
    async listAccounts() {
        return await this.rpc.getAllAccounts();
    }
    getContextEvents(account_id) {
        if (this.contextEmitters[account_id]) {
            return this.contextEmitters[account_id];
        }
        else {
            this.contextEmitters[account_id] = new TinyEmitter();
            return this.contextEmitters[account_id];
        }
    }
}
export const DEFAULT_OPTS = {
    url: "ws://localhost:20808/ws",
    startEventLoop: true,
};
export class DeltaChat extends BaseDeltaChat {
    close() {
        this.transport.close();
    }
    constructor(opts) {
        if (typeof opts === "string") {
            opts = Object.assign(Object.assign({}, DEFAULT_OPTS), { url: opts });
        }
        else if (opts) {
            opts = Object.assign(Object.assign({}, DEFAULT_OPTS), opts);
        }
        else {
            opts = Object.assign({}, DEFAULT_OPTS);
        }
        const transport = new WebsocketTransport(opts.url);
        super(transport, opts.startEventLoop);
        this.opts = opts;
    }
}
export class StdioDeltaChat extends BaseDeltaChat {
    close() { }
    constructor(input, output, startEventLoop) {
        const transport = new StdioTransport(input, output);
        super(transport, startEventLoop);
    }
}
export class StdioTransport extends BaseTransport {
    constructor(input, output) {
        super();
        this.input = input;
        this.output = output;
        var buffer = "";
        this.output.on("data", (data) => {
            buffer += data.toString();
            while (buffer.includes("\n")) {
                const n = buffer.indexOf("\n");
                const line = buffer.substring(0, n);
                const message = JSON.parse(line);
                this._onmessage(message);
                buffer = buffer.substring(n + 1);
            }
        });
    }
    _send(message) {
        const serialized = JSON.stringify(message);
        this.input.write(serialized + "\n");
    }
}
//# sourceMappingURL=client.js.map